@@ -4,17 +4,28 @@ from __future__ import annotations
 
 import re
 from contextlib import suppress
-from typing import TYPE_CHECKING, Any, List, Pattern, Tuple
+from typing import TYPE_CHECKING, List, Tuple
 
 from griffe.docstrings.dataclasses import (
-    DocstringAdmonition,
     DocstringAttribute,
     DocstringParameter,
     DocstringRaise,
     DocstringReceive,
     DocstringReturn,
     DocstringSection,
+    DocstringSectionAdmonition,
+    DocstringSectionAttributes,
+    DocstringSectionDeprecated,
+    DocstringSectionExamples,
     DocstringSectionKind,
+    DocstringSectionOtherParameters,
+    DocstringSectionParameters,
+    DocstringSectionRaises,
+    DocstringSectionReceives,
+    DocstringSectionReturns,
+    DocstringSectionText,
+    DocstringSectionWarns,
+    DocstringSectionYields,
     DocstringWarn,
     DocstringYield,
 )
@@ -22,6 +33,8 @@ from griffe.docstrings.utils import parse_annotation, warning
 from griffe.expressions import Expression, Name
 
 if TYPE_CHECKING:
+    from typing import Any, Literal, Pattern  # type: ignore[attr-defined]
+
     from griffe.dataclasses import Docstring
 
 _warn = warning(__name__)
@@ -32,13 +45,14 @@ _section_kind = {
     "params": DocstringSectionKind.parameters,
     "parameters": DocstringSectionKind.parameters,
     "keyword args": DocstringSectionKind.other_parameters,
-    "keyword parameters": DocstringSectionKind.other_parameters,
+    "keyword arguments": DocstringSectionKind.other_parameters,
     "other args": DocstringSectionKind.other_parameters,
     "other parameters": DocstringSectionKind.other_parameters,
     "raises": DocstringSectionKind.raises,
     "exceptions": DocstringSectionKind.raises,
     "returns": DocstringSectionKind.returns,
     "yields": DocstringSectionKind.yields,
+    "receives": DocstringSectionKind.receives,
     "examples": DocstringSectionKind.examples,
     "attributes": DocstringSectionKind.attributes,
 }
@@ -49,6 +63,8 @@ ItemsBlock = Tuple[BlockItems, int]
 
 _RE_ADMONITION: Pattern = re.compile(r"^(?P<type>[\w][\s\w-]*):(\s+(?P<title>[^\s].*))?$", re.I)
 _RE_NAME_ANNOTATION_DESCRIPTION: Pattern = re.compile(r"^(?:(?P<name>\w+)?\s*(?:\((?P<type>.+)\))?:\s*)?(?P<desc>.*)$")
+_RE_DOCTEST_BLANKLINE: Pattern = re.compile(r"^\s*<BLANKLINE>\s*$")
+_RE_DOCTEST_FLAGS: Pattern = re.compile(r"(\s*#\s*doctest:.+)$")
 
 
 def _read_block_items(docstring: Docstring, offset: int) -> ItemsBlock:  # noqa: WPS231
@@ -191,27 +207,39 @@ def _read_parameters(docstring: Docstring, offset: int) -> tuple[list[DocstringP
     return parameters, new_offset
 
 
-def _read_parameters_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:
+def _read_parameters_section(
+    docstring: Docstring,
+    offset: int,
+    **options: Any,
+) -> tuple[DocstringSectionParameters | None, int]:
     parameters, new_offset = _read_parameters(docstring, offset)
 
     if parameters:
-        return DocstringSection(DocstringSectionKind.parameters, parameters), new_offset
+        return DocstringSectionParameters(parameters), new_offset
 
     _warn(docstring, new_offset, f"Empty parameters section at line {offset}")
     return None, new_offset
 
 
-def _read_other_parameters_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:
+def _read_other_parameters_section(
+    docstring: Docstring,
+    offset: int,
+    **options: Any,
+) -> tuple[DocstringSectionOtherParameters | None, int]:
     parameters, new_offset = _read_parameters(docstring, offset)
 
     if parameters:
-        return DocstringSection(DocstringSectionKind.other_parameters, parameters), new_offset
+        return DocstringSectionOtherParameters(parameters), new_offset
 
     _warn(docstring, new_offset, f"Empty other parameters section at line {offset}")
     return None, new_offset
 
 
-def _read_attributes_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:  # noqa: WPS231
+def _read_attributes_section(
+    docstring: Docstring,
+    offset: int,
+    **options: Any,
+) -> tuple[DocstringSectionAttributes | None, int]:
     attributes = []
     block, new_offset = _read_block_items(docstring, offset)
 
@@ -240,13 +268,17 @@ def _read_attributes_section(docstring: Docstring, offset: int) -> tuple[Docstri
         attributes.append(DocstringAttribute(name=name, annotation=annotation, description=description))
 
     if attributes:
-        return DocstringSection(DocstringSectionKind.attributes, attributes), new_offset
+        return DocstringSectionAttributes(attributes), new_offset
 
     _warn(docstring, new_offset, f"Empty attributes section at line {offset}")
     return None, new_offset
 
 
-def _read_raises_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:
+def _read_raises_section(
+    docstring: Docstring,
+    offset: int,
+    **options: Any,
+) -> tuple[DocstringSectionRaises | None, int]:
     exceptions = []
     block, new_offset = _read_block_items(docstring, offset)
 
@@ -263,13 +295,17 @@ def _read_raises_section(docstring: Docstring, offset: int) -> tuple[DocstringSe
             exceptions.append(DocstringRaise(annotation=annotation, description=description))
 
     if exceptions:
-        return DocstringSection(DocstringSectionKind.raises, exceptions), new_offset
+        return DocstringSectionRaises(exceptions), new_offset
 
     _warn(docstring, new_offset, f"Empty exceptions section at line {offset}")
     return None, new_offset
 
 
-def _read_warns_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:
+def _read_warns_section(
+    docstring: Docstring,
+    offset: int,
+    **options: Any,
+) -> tuple[DocstringSectionWarns | None, int]:
     warns = []
     block, new_offset = _read_block_items(docstring, offset)
 
@@ -283,13 +319,17 @@ def _read_warns_section(docstring: Docstring, offset: int) -> tuple[DocstringSec
             warns.append(DocstringWarn(annotation=annotation, description=description))
 
     if warns:
-        return DocstringSection(DocstringSectionKind.warns, warns), new_offset
+        return DocstringSectionWarns(warns), new_offset
 
     _warn(docstring, new_offset, f"Empty warns section at line {offset}")
     return None, new_offset
 
 
-def _read_returns_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:  # noqa: WPS231
+def _read_returns_section(  # noqa: WPS231
+    docstring: Docstring,
+    offset: int,
+    **options: Any,
+) -> tuple[DocstringSectionReturns | None, int]:
     returns = []
     block, new_offset = _read_block_items(docstring, offset)
 
@@ -307,11 +347,24 @@ def _read_returns_section(docstring: Docstring, offset: int) -> tuple[DocstringS
             annotation = parse_annotation(annotation, docstring)
         else:
             # try to retrieve the annotation from the docstring parent
-            with suppress(AttributeError, KeyError):
+            with suppress(AttributeError, KeyError, ValueError):
                 annotation = docstring.parent.returns  # type: ignore[union-attr]
                 if len(block) > 1:
                     if annotation.is_tuple:
                         annotation = annotation.tuple_item(index)
+                    else:
+                        if annotation.is_iterator:
+                            return_item = annotation.iterator_item()
+                        elif annotation.is_generator:
+                            _, _, return_item = annotation.generator_items()
+                        else:
+                            raise ValueError
+                        if isinstance(return_item, Name):
+                            annotation = return_item
+                        elif return_item.is_tuple:
+                            annotation = return_item.tuple_item(index)
+                        else:
+                            annotation = return_item
 
             if annotation is None:
                 returned_value = repr(name) or index
@@ -320,13 +373,17 @@ def _read_returns_section(docstring: Docstring, offset: int) -> tuple[DocstringS
         returns.append(DocstringReturn(name=name or "", annotation=annotation, description=description))
 
     if returns:
-        return DocstringSection(DocstringSectionKind.returns, returns), new_offset
+        return DocstringSectionReturns(returns), new_offset
 
     _warn(docstring, new_offset, f"Empty returns section at line {offset}")
     return None, new_offset
 
 
-def _read_yields_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:  # noqa: WPS231
+def _read_yields_section(  # noqa: WPS231
+    docstring: Docstring,
+    offset: int,
+    **options: Any,
+) -> tuple[DocstringSectionYields | None, int]:
     yields = []
     block, new_offset = _read_block_items(docstring, offset)
 
@@ -339,17 +396,26 @@ def _read_yields_section(docstring: Docstring, offset: int) -> tuple[DocstringSe
         name, annotation, description = match.groups()
         description = "\n".join([description.lstrip(), *yield_lines[1:]]).rstrip("\n")
 
-        if annotation and docstring.parent is not None:
+        if annotation:
             # try to compile the annotation to transform it into an expression
             annotation = parse_annotation(annotation, docstring)
         else:
             # try to retrieve the annotation from the docstring parent
-            with suppress(AttributeError, KeyError):
+            with suppress(AttributeError, KeyError, ValueError):
                 annotation = docstring.parent.returns  # type: ignore[union-attr]
-                # TODO: support getting yield part and exploding tuple (in a generator/iterator)
-                # if len(block) > 1:
-                #     if annotation.is_tuple:
-                #         annotation = annotation.tuple_item(index)
+                if len(block) > 1:
+                    if annotation.is_iterator:
+                        yield_item = annotation.iterator_item()
+                    elif annotation.is_generator:
+                        yield_item, _, _ = annotation.generator_items()
+                    else:
+                        raise ValueError
+                    if isinstance(yield_item, Name):
+                        annotation = yield_item
+                    elif yield_item.is_tuple:
+                        annotation = yield_item.tuple_item(index)
+                    else:
+                        annotation = yield_item
 
             if annotation is None:
                 yielded_value = repr(name) or index
@@ -358,13 +424,17 @@ def _read_yields_section(docstring: Docstring, offset: int) -> tuple[DocstringSe
         yields.append(DocstringYield(name=name or "", annotation=annotation, description=description))
 
     if yields:
-        return DocstringSection(DocstringSectionKind.yields, yields), new_offset
+        return DocstringSectionYields(yields), new_offset
 
     _warn(docstring, new_offset, f"Empty yields section at line {offset}")
     return None, new_offset
 
 
-def _read_receives_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:  # noqa: WPS231
+def _read_receives_section(  # noqa: WPS231
+    docstring: Docstring,
+    offset: int,
+    **options: Any,
+) -> tuple[DocstringSectionReceives | None, int]:
     receives = []
     block, new_offset = _read_block_items(docstring, offset)
 
@@ -380,14 +450,18 @@ def _read_receives_section(docstring: Docstring, offset: int) -> tuple[Docstring
         if annotation:
             # try to compile the annotation to transform it into an expression
             annotation = parse_annotation(annotation, docstring)
-        # else:
-        # try to retrieve the annotation from the docstring parent
-        # TODO: support getting receive part and exploding tuple (in a generator/iterator)
-        # with suppress(AttributeError, KeyError):
-        #     annotation = docstring.parent.returns  # type: ignore[union-attr]
-        #     if len(block) > 1:
-        #         if annotation.is_tuple:
-        #             annotation = annotation.tuple_item(index)
+        else:
+            # try to retrieve the annotation from the docstring parent
+            with suppress(AttributeError, KeyError):
+                annotation = docstring.parent.returns  # type: ignore[union-attr]
+                if len(block) > 1 and annotation.is_generator:
+                    _, receives_item, _ = annotation.generator_items()
+                    if isinstance(receives_item, Name):
+                        annotation = receives_item
+                    elif receives_item.is_tuple:
+                        annotation = receives_item.tuple_item(index)
+                    else:
+                        annotation = receives_item
 
         if annotation is None:
             received_value = repr(name) or index
@@ -396,16 +470,21 @@ def _read_receives_section(docstring: Docstring, offset: int) -> tuple[Docstring
         receives.append(DocstringReceive(name=name or "", annotation=annotation, description=description))
 
     if receives:
-        return DocstringSection(DocstringSectionKind.receives, receives), new_offset
+        return DocstringSectionReceives(receives), new_offset
 
     _warn(docstring, new_offset, f"Empty receives section at line {offset}")
     return None, new_offset
 
 
-def _read_examples_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:  # noqa: WPS231
+def _read_examples_section(  # noqa: WPS231
+    docstring: Docstring,
+    offset: int,
+    trim_doctest_flags: bool = True,
+    **options: Any,
+) -> tuple[DocstringSectionExamples | None, int]:
     text, new_offset = _read_block(docstring, offset)
 
-    sub_sections = []
+    sub_sections: list[tuple[Literal[DocstringSectionKind.text] | Literal[DocstringSectionKind.examples], str]] = []
     in_code_example = False
     in_code_block = False
     current_text: list[str] = []
@@ -422,6 +501,9 @@ def _read_examples_section(docstring: Docstring, offset: int) -> tuple[Docstring
                 current_text.append(line)
 
         elif in_code_example:
+            if trim_doctest_flags:
+                line = _RE_DOCTEST_FLAGS.sub("", line)
+                line = _RE_DOCTEST_BLANKLINE.sub("", line)
             current_example.append(line)
 
         elif line.startswith("```"):
@@ -436,6 +518,9 @@ def _read_examples_section(docstring: Docstring, offset: int) -> tuple[Docstring
                 sub_sections.append((DocstringSectionKind.text, "\n".join(current_text).rstrip("\n")))
                 current_text = []
             in_code_example = True
+
+            if trim_doctest_flags:
+                line = _RE_DOCTEST_FLAGS.sub("", line)
             current_example.append(line)
 
         else:
@@ -447,13 +532,17 @@ def _read_examples_section(docstring: Docstring, offset: int) -> tuple[Docstring
         sub_sections.append((DocstringSectionKind.examples, "\n".join(current_example)))
 
     if sub_sections:
-        return DocstringSection(DocstringSectionKind.examples, sub_sections), new_offset
+        return DocstringSectionExamples(sub_sections), new_offset
 
     _warn(docstring, new_offset, f"Empty examples section at line {offset}")
     return None, new_offset
 
 
-def _read_deprecated_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:
+def _read_deprecated_section(
+    docstring: Docstring,
+    offset: int,
+    **options: Any,
+) -> tuple[DocstringSectionDeprecated | None, int]:
     text, new_offset = _read_block(docstring, offset)
 
     # early exit if there is no text in the yield section
@@ -472,7 +561,7 @@ def _read_deprecated_section(docstring: Docstring, offset: int) -> tuple[Docstri
     description = text.lstrip()
 
     return (
-        DocstringSection(DocstringSectionKind.deprecated, (version, description)),
+        DocstringSectionDeprecated(version=version, text=description),
         new_offset,
     )
 
@@ -498,6 +587,7 @@ _section_reader = {
 def parse(  # noqa: WPS231
     docstring: Docstring,
     ignore_init_summary: bool = False,
+    trim_doctest_flags: bool = True,
     **options: Any,
 ) -> list[DocstringSection]:
     """Parse a docstring.
@@ -508,19 +598,26 @@ def parse(  # noqa: WPS231
     Parameters:
         docstring: The docstring to parse.
         ignore_init_summary: Whether to ignore the summary in `__init__` methods' docstrings.
+        trim_doctest_flags: Whether to remove doctest flags from Python example blocks.
         **options: Additional parsing options.
 
     Returns:
         A list of docstring sections.
     """
-    sections = []
+    sections: list[DocstringSection] = []
     current_section = []
 
     in_code_block = False
     lines = docstring.lines
 
+    options = {
+        "ignore_init_summary": ignore_init_summary,
+        "trim_doctest_flags": trim_doctest_flags,
+        **options,
+    }
+
     ignore_summary = (
-        ignore_init_summary  # noqa: WPS222
+        options["ignore_init_summary"]  # noqa: WPS222
         and docstring.parent is not None
         and docstring.parent.name == "__init__"
         and docstring.parent.is_function
@@ -555,15 +652,10 @@ def parse(  # noqa: WPS231
                 if admonition_type.lower() in _section_kind:
                     if current_section:
                         if any(current_section):
-                            sections.append(
-                                DocstringSection(
-                                    DocstringSectionKind.text,
-                                    "\n".join(current_section).rstrip("\n"),
-                                )
-                            )
+                            sections.append(DocstringSectionText("\n".join(current_section).rstrip("\n")))
                         current_section = []
                     reader = _section_reader[_section_kind[admonition_type.lower()]]
-                    section, offset = reader(docstring, offset + 1)
+                    section, offset = reader(docstring, offset + 1, **options)  # type: ignore[operator]
                     if section:
                         section.title = title
                         sections.append(section)
@@ -571,16 +663,14 @@ def parse(  # noqa: WPS231
                 else:
                     contents, offset = _read_block(docstring, offset + 1)
                     if contents:
+                        if current_section:
+                            if any(current_section):
+                                sections.append(DocstringSectionText("\n".join(current_section).rstrip("\n")))
+                            current_section = []
                         if title is None:
                             title = admonition_type
                         admonition_type = admonition_type.lower().replace(" ", "-")
-                        sections.append(
-                            DocstringSection(
-                                kind=DocstringSectionKind.admonition,
-                                value=DocstringAdmonition(kind=admonition_type, contents=contents),
-                                title=title,
-                            )
-                        )
+                        sections.append(DocstringSectionAdmonition(kind=admonition_type, text=contents, title=title))
                     else:
                         with suppress(IndexError):
                             current_section.append(lines[offset])
@@ -590,6 +680,6 @@ def parse(  # noqa: WPS231
         offset += 1
 
     if current_section:
-        sections.append(DocstringSection(DocstringSectionKind.text, "\n".join(current_section).rstrip("\n")))
+        sections.append(DocstringSectionText("\n".join(current_section).rstrip("\n")))
 
     return sections
