@@ -6,8 +6,10 @@ import inspect
 
 import pytest
 
-from griffe.dataclasses import Class, Function, Module, Parameter, Parameters
+from griffe.dataclasses import Attribute, Class, Docstring, Function, Module, Parameter, Parameters
 from griffe.docstrings.dataclasses import DocstringSectionKind
+from griffe.docstrings.utils import parse_annotation
+from griffe.expressions import Name
 
 
 # =============================================================================================
@@ -735,6 +737,32 @@ def test_parameter_line_without_colon_keyword_only(parse_google):
 #     assert not warnings
 
 
+# =============================================================================================
+# Attributes sections
+def test_retrieve_attributes_annotation_from_parent(parse_google):
+    """Retrieve the annotations of attributes from the parent object.
+
+    Parameters:
+        parse_google: Fixture parser.
+    """
+    docstring = """
+        Summary.
+
+        Attributes:
+            a: Whatever.
+            b: Whatever.
+    """
+    parent = Class("cls")
+    parent["a"] = Attribute("a", annotation=Name("int", "int"))
+    parent["b"] = Attribute("b", annotation=Name("str", "str"))
+    sections, _ = parse_google(docstring, parent=parent)
+    attributes = sections[1].value
+    assert attributes[0].name == "a"
+    assert attributes[0].annotation.source == "int"
+    assert attributes[1].name == "b"
+    assert attributes[1].annotation.source == "str"
+
+
 # =============================================================================================
 # Yields sections
 def test_parse_yields_section_with_return_annotation(parse_google):
@@ -757,6 +785,99 @@ def test_parse_yields_section_with_return_annotation(parse_google):
     assert not warnings
 
 
+@pytest.mark.parametrize(
+    "return_annotation",
+    [
+        "Iterator[tuple[int, float]]",
+        "Generator[tuple[int, float], ..., ...]",
+    ],
+)
+def test_parse_yields_tuple_in_iterator_or_generator(parse_google, return_annotation):
+    """Parse Yields annotations in Iterator or Generator types.
+
+    Parameters:
+        parse_google: Fixture parser.
+        return_annotation: Parametrized return annotation as a string.
+    """
+    docstring = """
+        Summary.
+
+        Yields:
+            a: Whatever.
+            b: Whatever.
+    """
+    sections, _ = parse_google(
+        docstring,
+        parent=Function(
+            "func",
+            returns=parse_annotation(return_annotation, Docstring("d", parent=Function("f"))),
+        ),
+    )
+    yields = sections[1].value
+    assert yields[0].name == "a"
+    assert yields[0].annotation.source == "int"
+    assert yields[1].name == "b"
+    assert yields[1].annotation.source == "float"
+
+
+# =============================================================================================
+# Receives sections
+def test_parse_receives_tuple_in_generator(parse_google):
+    """Parse Receives annotations in Generator type.
+
+    Parameters:
+        parse_google: Fixture parser.
+    """
+    docstring = """
+        Summary.
+
+        Receives:
+            a: Whatever.
+            b: Whatever.
+    """
+    sections, _ = parse_google(
+        docstring,
+        parent=Function(
+            "func",
+            returns=parse_annotation("Generator[..., tuple[int, float], ...]", Docstring("d", parent=Function("f"))),
+        ),
+    )
+    receives = sections[1].value
+    assert receives[0].name == "a"
+    assert receives[0].annotation.source == "int"
+    assert receives[1].name == "b"
+    assert receives[1].annotation.source == "float"
+
+
+# =============================================================================================
+# Returns sections
+def test_parse_returns_tuple_in_generator(parse_google):
+    """Parse Returns annotations in Generator type.
+
+    Parameters:
+        parse_google: Fixture parser.
+    """
+    docstring = """
+        Summary.
+
+        Returns:
+            a: Whatever.
+            b: Whatever.
+    """
+    sections, _ = parse_google(
+        docstring,
+        parent=Function(
+            "func",
+            returns=parse_annotation("Generator[..., ..., tuple[int, float]]", Docstring("d", parent=Function("f"))),
+        ),
+    )
+    returns = sections[1].value
+    assert returns[0].name == "a"
+    assert returns[0].annotation.source == "int"
+    assert returns[1].name == "b"
+    assert returns[1].annotation.source == "float"
+
+
 # =============================================================================================
 # Parser special features
 def test_parse_admonitions(parse_google):
@@ -826,6 +947,26 @@ def test_handle_false_admonitions_correctly(parse_google, docstring):
     assert not warnings
 
 
+def test_dont_insert_admonition_before_current_section(parse_google):
+    """Check that admonitions are inserted at the right place.
+
+    Parameters:
+        parse_google: Fixture parser.
+    """
+    docstring = """
+        Summary.
+
+        Short description.
+
+        Info:
+            Something useful.
+    """
+    sections, _ = parse_google(docstring)
+    assert len(sections) == 2
+    assert sections[0].kind is DocstringSectionKind.text
+    assert sections[1].kind is DocstringSectionKind.admonition
+
+
 @pytest.mark.parametrize(
     "docstring",
     [
